@@ -1,7 +1,7 @@
 #
 # bootmisc.sh	Miscellaneous things to be done during bootup.
 #
-# Version:	@(#)bootmisc.sh  1.10  22-Jun-1998  miquels@cistron.nl
+# Version:	@(#)bootmisc.sh  1.20  09-Jan-1999  miquels@cistron.nl
 #
 
 . /etc/default/rcS
@@ -20,9 +20,20 @@ fi
 # Note that files _in_ lost+found _are_ deleted.
 #
 [ "$VERBOSE" != no ] && echo -n "Cleaning: /tmp "
+#
+#	If $TMPTIME is set to 0, we do not use any ctime expression
+#	at all, so we can also delete files with timestamps
+#	in the future!
+#
+if [ "$TMPTIME" = 0 ]
+then
+	TEXPR=""
+else
+	TEXPR="! -ctime -$TMPTIME"
+fi
 ( cd /tmp && \
   find . -xdev \
-  ! -ctime -$TMPTIME \
+  $TEXPR \
   ! -name . \
   ! \( -name lost+found -uid 0 \) \
   ! \( -name quota.user -uid 0 \) \
@@ -47,8 +58,11 @@ rm -f /tmp/.X*-lock
 #
 # Set pseudo-terminal access permissions.
 #
-chmod 666 /dev/tty[p-za-e][0-9a-f]
-chown root:tty /dev/tty[p-za-e][0-9a-f]
+if [ -c /dev/ttyp0 ]
+then
+	chmod 666 /dev/tty[p-za-e][0-9a-f]
+	chown root:tty /dev/tty[p-za-e][0-9a-f]
+fi
 
 #
 # Update /etc/motd.
