@@ -1,14 +1,14 @@
 #
 # checkroot.sh	Check to root file system.
 #
-# Version:	@(#)checkroot.sh  1.10  22-Jun-1998  miquels@cistron.nl
+# Version:	@(#)checkroot.sh  2.76  12-Jan-1999  miquels@cistron.nl
 #
 
 . /etc/default/rcS
 
 #
-# Set SULOGIN to yes if you want a sulogin to be spawned from
-# this script *before anything else* with a timeout, like on SCO.
+# Set SULOGIN in /etc/default/rcS to yes if you want a sulogin to be spawned
+# from this script *before anything else* with a timeout, like SCO does.
 #
 [ "$SULOGIN" = yes ] && sulogin -t 30 $CONSOLE
 
@@ -47,7 +47,7 @@ else
     else
 	force=""
     fi
-    if [ "$FSCKFIX" ]
+    if [ "$FSCKFIX" = yes ]
     then
 	fix="-y"
     else
@@ -86,13 +86,41 @@ else
 fi
 
 #
-# Remount rootfs rw (default), but do not try to change mtab because it
-# is on a ro fs until the remount succeeded.  Then clean up old mtabs and
-# finally write the new mtab.
+#	If the root filesystem was not marked as read-only in /etc/fstab,
+#	remount the rootfs rw but do not try to change mtab because it
+#	is on a ro fs until the remount succeeded. Then clean up old mtabs
+#	and finally write the new mtab.
 #
-mount -n -o remount,rw /
-rm -f /etc/mtab~ /etc/nologin
-: > /etc/mtab
-mount -o remount,rw /
-mount /proc
+(
+  mode=rw
+  rootopts=rw
+  devfs=
+  while read fs mnt type opts rest
+  do
+	case "$fs" in
+		""|\#*)
+			continue;
+			;;
+	esac
+	[ "$type" = devfs ] && devfs="$fs"
+	[ "$mnt" != / ] && continue
+	rootopts="$opts"
+	case "$opts" in
+		ro|ro,*|*,ro|*,ro,*)
+			mode=ro
+			;;
+	esac
+  done
+  mount -n -o remount,$rootopts /
+  if [ "$mode" = rw ]
+  then
+	rm -f /etc/mtab~ /etc/nologin
+	: > /etc/mtab
+	mount -f -o remount,$rootopts /
+	mount /proc
+	[ "$devfs" ] && grep -q '^devfs /dev' /proc/mounts && mount -f "$devfs"
+  else
+	mount -n /proc
+  fi
+) < /etc/fstab
 
