@@ -5,10 +5,10 @@
  *		     init [0123456SsQqAaBbCc]
  *		  telinit [0123456SsQqAaBbCc]
  *
- * Version:	@(#)init.c  2.76-1  03-Nov-1998  miquels@cistron.nl
+ * Version:	@(#)init.c  2.76-4  24-Apr-1999  miquels@cistron.nl
  *
  *		This file is part of the sysvinit suite,
- *		Copyright 1991-1998 Miquel van Smoorenburg.
+ *		Copyright 1991-1999 Miquel van Smoorenburg.
  *
  *		This program is free software; you can redistribute it and/or
  *		modify it under the terms of the GNU General Public License
@@ -45,6 +45,15 @@
 #include <sys/syslog.h>
 #include <sys/time.h>
 
+#ifdef __i386__
+#  if (__GLIBC__ >= 2)
+#    include <sigcontext.h>
+#  else
+#    include <asm/sigcontext.h>
+#    define sigcontext sigcontext_struct
+#  endif
+#endif
+
 #include "init.h"
 #include "initreq.h"
 #include "paths.h"
@@ -61,7 +70,7 @@
 				sigaction(sig, &sa, NULL);
 
 /* Version information */
-char *Version = "@(#) init 2.76-1  03-Nov-1998  miquels@cistron.nl";
+char *Version = "@(#) init 2.76-4  24-Apr-1999  miquels@cistron.nl";
 char *bootmsg = "version 2.76 booting";
 #define E_VERSION "INIT_VERSION=sysvinit-2.76"
 
@@ -94,6 +103,7 @@ char *console_dev;		/* Console device. */
 int wrote_iosave = 0;		/* Did we write ioctl.save yet? */
 int pipe_fd = -1;		/* /dev/initctl */
 int did_boot = 0;		/* Did we already do BOOT* stuff? */
+int main(int, char **);
 
 /*	Used by re-exec part */
 int reload = 0;			/* Should we do initialization stuff? */
@@ -536,12 +546,27 @@ void cont_handler()
 
 /*
  *	OOPS: segmentation violation!
+ *	If we have the info, print where it occured.
+ *	Then sleep 30 seconds and try to continue.
  */
+#ifdef __i386__
+void segv_handler(int sig, struct sigcontext ctx)
+{
+	char	*p = "";
+
+	if ((void *)ctx.eip >= (void *)do_sleep &&
+	    (void *)ctx.eip < (void *)main)
+		p = " (code)";
+	log(L_VB, "PANIC: segmentation violation at %p%s! sleeping for 30 seconds.", p);
+	do_sleep(30);
+}
+#else
 void segv_handler()
 {
-	log(L_VB, "PANIC: segmentation violation! giving up..");
-	while(1) pause();
+	log(L_VB, "PANIC: segmentation violation! sleeping for 30 seconds.");
+	do_sleep(30);
 }
+#endif
 
 /*
  *	The SIGSTOP & SIGTSTP handler
@@ -1845,6 +1870,7 @@ void check_init_fifo(void)
   fd_set fds;
   int quit = 0;
   struct stat st, st2;
+  struct timeval tv;
 
   /*
    *	First, try to create /dev/initctl if not present.
@@ -1879,10 +1905,16 @@ void check_init_fifo(void)
 		}
 	}
 	if (pipe_fd >= 0) {
-		/* Don't use fd's 0, 1 or 2. */
+		/*
+		 *	Don't use fd's 0, 1 or 2.
+		 */
 		(void) dup2(pipe_fd, PIPE_FD);
 		close(pipe_fd);
 		pipe_fd = PIPE_FD;
+
+		/*
+		 *	Return to caller - we'll be back later.
+		 */
 	}
   }
 
@@ -1892,9 +1924,11 @@ void check_init_fifo(void)
 	/* Do select, return on EINTR. */
 	FD_ZERO(&fds);
 	FD_SET(pipe_fd, &fds);
-	n = select(pipe_fd + 1, &fds, NULL, NULL, NULL);
+	tv.tv_sec = 5;
+	tv.tv_usec = 0;
+	n = select(pipe_fd + 1, &fds, NULL, NULL, &tv);
 	if (n <= 0) {
-		if (errno == EINTR) return;
+		if (n == 0 || errno == EINTR) return;
 		continue;
 	}
 
@@ -2232,7 +2266,7 @@ int init_main()
   SETSIG(sa, SIGSTOP,  stop_handler);
   SETSIG(sa, SIGTSTP,  stop_handler);
   SETSIG(sa, SIGCONT,  cont_handler);
-  SETSIG(sa, SIGSEGV,  segv_handler);
+  SETSIG(sa, SIGSEGV,  (void(*)(int))segv_handler);
 
   /*
    *	NOTE: instead of calling siginterrupt, we could just as
